import json

from django import forms
from django.conf import settings
from django.utils.safestring import mark_safe
from django.apps import apps


from .settings import YAMAPS_ADDRESS_MODEL


class AddressWidget(forms.TextInput):

    parts = ["latitude", "longitude", "country",
             "admin_area", "locality", "street", "house"]

    def __init__(self, *args, **kwargs):
        attrs = kwargs.get('attrs', {})
        classes = attrs.get('class', '')
        classes += (' ' if classes else '') + 'yamaps'
        attrs['class'] = classes
        kwargs['attrs'] = attrs
        super(AddressWidget, self).__init__(*args, **kwargs)

    def _media(self):
        lang = "ru-RU"
        if settings.LANGUAGE_CODE:
            lang = settings.LANGUAGE_CODE
        yamaps_js_url = \
            "https://api-maps.yandex.ru/2.1/?lang={}".format(lang)
        return forms.Media(js=(yamaps_js_url, "yamaps/js/yamaps-widget.js"))

    media = property(_media)

    def render(self, name, value, attrs=None, **kwargs):
        # Raw address input will be readonly.
        if "readonly" not in attrs:
            attrs["readonly"] = "readonly"

        raw_address = ""

        if isinstance(value, int):
            addr_model = apps.get_model(YAMAPS_ADDRESS_MODEL)
            value = addr_model.objects.get(pk=value)
            try:
                raw_address = value.to_raw()
            except AttributeError:
                raw_address = str(value)

        text_input = super().render(name, raw_address, attrs, **kwargs)

        widget = """{old}
<div id="{name}_components">
<input type="hidden" name="{name}_json"/>
<div id="{name}_map" style="width: 700px; height: 500px;"></div>
</div>
        """.format(old=text_input, name=name)

        return mark_safe(widget)

    def value_from_datadict(self, data, files, name):
        raw = data.get(name, None)
        if not raw:
            return raw

        addr_json = data.get("{}_json".format(name))
        addr = json.loads(addr_json)
        addr["raw"] = raw
        return addr



from django import template
from django.conf import settings
from django.utils.safestring import mark_safe

register = template.Library()


@register.simple_tag
def yamaps(raw_address, style="width: 480px; height: 320px;"):
    return mark_safe('<div class="django-yamaps" data-raw-address="{}" style="{}"></div>'.format(raw_address, style))


@register.simple_tag
def yamaps_init():
    lang = "ru-RU"
    if settings.LANGUAGE_CODE:
        lang = settings.LANGUAGE_CODE
    yamaps_js = '<script type="text/javascript" src="https://api-maps.yandex.ru/2.1/?lang={}"></script>' \
        .format(lang)
    return mark_safe(yamaps_js + '''
    <script type="text/javascript">
    window.onload = function() {
        ymaps.ready(function () {
            var maps = document.querySelectorAll(".django-yamaps");
            maps.forEach(function (elem) {
                var map = new ymaps.Map(elem, {
                    center: [53.758768, 87.136731],
                    zoom: 15
                });
                var addr = elem.getAttribute("data-raw-address");
                ymaps.geocode(addr, { results: 1 }).then(function (res) {
                    var firstGeoObject = res.geoObjects.get(0),
                        coords = firstGeoObject.geometry.getCoordinates();
                    map.geoObjects.add(firstGeoObject);
                    map.setCenter(coords);
                });
            });
    })};
    </script>''')



import os
import sys
from setuptools import setup, find_packages

version = '0.3.0'

if sys.argv[-1] == 'tag':
    print("Tagging the version on github:")
    os.system("git tag -a %s -m 'version %s'" % (version, version))
    os.system("git push --tags")
    sys.exit()

setup(
    name='django-yamaps',
    version=version,
    author='herald-it',
    author_email='wessmoke@gmail.com',
    maintainer='Dmitry Shlagov',
    maintainer_email='wessmoke@gmail.com',

    url='https://github.com/herald-it/django-yamaps',
    download_url='https://github.com/herald-it/django-yamaps/archive/%s.tar.gz' % version,

    description='A django widget which allows you to select the address using the yandex map.',
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.md')).read(),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: Russian',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5'
    ],
    license='MIT',

    packages=find_packages(),
    include_package_data=True,
    package_data={'': ['*.js', '*.html', '*.*']},
    install_requires=['setuptools'],
    zip_safe=False,
    keywords=['django', 'address', 'yandex', 'maps', 'admin']
)
